SUBROUTINE sub_poloffgrid(c_grid,s_grid,val_grid,a,h_out,c,s)
  ! Computes interpolated (unconditional) policy functions off nodes
  ! given consumption, saving, and value fns on nodes
  ! CONDITIONAL on housing choice
  ! Inputs:
  !         Functions on interpolating nodes
  !	        c_grid, s_grid, val_grid, a
  ! Outputs:
  !         Interpolated value for policy fns
  !         h_out, c, s
	
  USE mod_types
  USE mod_common_var
  USE mod_int_node

  IMPLICIT NONE

  REAL(dp), INTENT(in), DIMENSION(len_a_grid,len_h_grid):: &
      c_grid, s_grid, val_grid
  REAL(dp), INTENT(in)			:: a
  INTEGER, INTENT(out) 			:: h_out
  REAL(dp), INTENT(out)			:: c, s

  REAL(dp), DIMENSION(len_h_grid)	:: val
  INTEGER				:: index_h_prime, node
  
  ! Value function conditional on housing choice index_h_prime
  DO index_h_prime = 1, len_h_grid
    ! Recover RIGHT interpolating nodes
    CALL sub_int_node(a_grid,a,node)
    node = node - 1
    val(index_h_prime) = val_grid(node,index_h_prime) + &
        ( val_grid(node+1,index_h_prime) - val_grid(node,index_h_prime) )/ &
        ( a_grid(node+1) - a_grid(node) ) * ( a - a_grid(node) )
  END DO

  ! Current housing choice 
  h_out = MAXLOC(val,1)

  ! Current consumption and saving
  c = c_grid(node,h_out) + &
      ( c_grid(node+1,h_out) - c_grid(node,h_out) )/ &
      ( a_grid(node+1) - a_grid(node) ) * ( a - a_grid(node) )
  s = s_grid(node,h_out) + &
      ( s_grid(node+1,h_out) - s_grid(node,h_out) )/ &
      ( a_grid(node+1) - a_grid(node) ) * ( a - a_grid(node) )

  ! Note to self: saving could also be obtained using the budget
  ! constrained
  
END SUBROUTINE sub_poloffgrid
