SUBROUTINE sub_rnumderiv(X,Y,dY_dX)
! Calculates numerical RIGHT derivative at point X_i using the finite 
! difference formula (Y_(i+1)-Y(i))/(X_(i+1)-X(i)).
! The last two values are identical.

    USE mod_types
    IMPLICIT NONE


    REAL(dp), DIMENSION(:),INTENT(in) :: X,Y
    REAL(dp), DIMENSION(SIZE(X)),INTENT(out) :: dY_dX
    REAL(dp), DIMENSION(size(X)) :: X_plus,dX,Y_plus,dY
    
    INTEGER :: n

    n = size(X)

    X_plus  = (/ X(2:n), X(n) /)
    dX = X_plus - X
    dX(n) = dX(n-1)
    if (any(dX==0.d0)) print *, "Division by zero in rnumderiv"
    
    Y_plus  = (/ Y(2:n), Y(n) /)
    dY = Y_plus - Y
    dY(n) = dY(n-1)
    
    dY_dX = dY/dX 
    continue
END SUBROUTINE sub_rnumderiv
