SUBROUTINE sub_tauchen(y,pi,picum,rho,mu0,sigma,n,cover_tauchen)
	! Calculates Tauchen points for AR(1) x=(1-rho)mu0 + rho*x_(-1) + u
	! Adapted from sub_tauchen by Jesus Fernandez-Villaverde (undated)
	! cover_tauchen is the number of SD (each side) that one wants
	! to cover.
 
	USE mod_types

	IMPLICIT NONE

	INTEGER, INTENT(in) :: n

	REAL(dp), INTENT(in) :: rho, mu0, sigma, cover_tauchen
	REAL(dp), DIMENSION (n,n), INTENT(out) :: pi , picum
	REAL(dp), DIMENSION (n), INTENT(out) :: y

	INTEGER :: yc, tc, tcc,i1

	REAL(dp) :: sigy, mu, upp, low
	REAL(dp), EXTERNAL  :: cum_normal

	! Define the discrete states of the Markov chain

	sigy = sigma/SQRT(1.0-rho**2)

	y(n) = mu0 + cover_tauchen*sigy
	y(1) = mu0 - cover_tauchen*sigy

	DO yc = 2, n-1

		y(yc) = y(1)+(y(n)-y(1))*(yc-1.0)/(n-1.0)

	END DO

	! Compute the transition matrix

	DO tc = 1,n
		mu = (1-rho)*mu0 + rho*y(tc)
		upp = (y(2)+y(1))/2.0
		upp = (upp-mu)/sigma
		pi(tc,1) = cum_normal(upp)
		low = (y(n)+y(n-1))/2.0
		low = (low-mu)/sigma
		pi(tc,n) = 1.d0-cum_normal(low)

		DO tcc = 2, n-1

			low = (y(tcc-1)+y(tcc))/2.0
			upp = (y(tcc+1)+y(tcc))/2.0
			low = (low-mu)/sigma
			upp = (upp-mu)/sigma
			pi(tc,tcc) = cum_normal(upp)-cum_normal(low)

		END DO
	END DO

	! Compute the CDF of the transition matrix.
	picum(:,1)=pi(:,1)
	DO tc = 2,n
		picum(:,tc)=picum(:,tc-1)+pi(:,tc)
	ENDDO

	OPEN(unit=98,file='Tauchen.txt')
	WRITE(98,*) 'rho, sigma, n'
	WRITE(98,*) rho, sigma, n
	WRITE(98,*) 'Income States'
	WRITE(98,*) y(1:n)
	WRITE(98,*) 'Transition Matrix'

	DO tc=1,n
		WRITE(98,*) pi(tc,:)
	END DO

	WRITE(98,*) 'Transition Matrix CDF'

	DO tc=1,n
		WRITE(98,*) picum(tc,:)
	END DO

	CLOSE(98)

	OPEN(unit=16,	file='transition.txt', status = 'replace')
	WRITE (16, '(9f20.10)') (pi(i1, :), i1 = 1,n)
	CLOSE(16)


END SUBROUTINE sub_tauchen
