  SUBROUTINE &
      sub_valueiter(index_house,con,sav,val_choice,val_in,dval_in)
  ! Iterates  over  value function using my generalized egm
  ! to solve for the optimal consumption/saving functions
  ! Inputs:
  !   	val_in, dval_in: guesses for value fn and its wealth derivative
  ! Outputs (policy and value fns on the EXOGENOUS asset grid):
  ! 	index_house   : housing choice policy fn 
  !	con, sav and  val_choice: consumption, saving and value fns
  !	CONDITIONAL on housing choice.
  !     They are a function of the state AND the current housing choice 
  !
  ! If eg_flag = 1 it also stores consumption and value fns on the 
  ! ENDOGENOUS cash-at-hand grid
  USE mod_types
  USE mod_common_var  
  USE mod_interface, ONLY: sub_rnumderiv
  USE mod_felicity

  IMPLICIT NONE

  REAL (dp), INTENT(in), DIMENSION(len_a_grid,len_h_grid,len_y_grid):: &
      val_in, dval_in
  REAL (dp), INTENT(out), DIMENSION(len_a_grid,len_h_grid,len_h_grid,&
      len_y_grid) 	:: con, sav, val_choice
  INTEGER, INTENT(out), DIMENSION(len_a_grid,len_h_grid,len_y_grid):: &
      index_house

  REAL (dp), DIMENSION(len_a_grid,len_h_grid,len_y_grid):: deriv, deriv2,&
      last_val, exp_dvalue, dval,uncon_sav
  REAL (dp), DIMENSION(len_a_grid,len_h_grid,len_y_grid):: exp_value, val 
  REAL (dp), DIMENSION(len_a_grid,len_h_grid,len_h_grid,len_y_grid):: &
      z_grid
  REAL (dp), DIMENSION(len_a_grid) :: c_node, exp_val, exp_dval, &
      z_node,val_node
  REAL (dp)     :: diff, max_value, trans_cost, val_temp, c_real
  REAL (dp), EXTERNAL :: z
  REAL          :: t1, t2

  INTEGER	:: index, index_a, index_h, index_h_prime, index_y,iter,j
  INTEGER	:: i, k, last_max_a, max_loc


  iter = 0
  diff = 100.d0; val = val_in; dval = dval_in

  IF (eg_flag==1) THEN
    ! Allocates global variables for policy and value fns on the 
    ! endogenous cash-at-hand grid
    ALLOCATE(max_gr(len_h_grid,len_y_grid))
    ALLOCATE(c_gr(len_a_grid,len_h_grid,len_y_grid))
    ALLOCATE(val_gr(len_a_grid,len_h_grid,len_y_grid))
    ALLOCATE(zend_gr(len_a_grid,len_h_grid,len_y_grid))
    max_gr = 0
    c_gr = 0.d0; val_gr = 0.d0; zend_gr = 0.d0
  END IF
  

  !Interpolating nodes for cash at hand
  !------------------------------------
  DO index_y = 1,len_y_grid          
    DO index_h = 1,len_h_grid  
      DO index_h_prime = 1, len_h_grid
        DO index_a = 1, len_a_grid    
          z_grid(index_a,index_h_prime,index_h,index_y) = &
          z(a_grid(index_a),index_h_prime,index_h,index_y)
        END DO
      END DO
    END DO
  END DO


  t1 = secnds(0.0)

  DO iter = 1,10000  ! do 0
    last_val = val
    con = 1.d-7
    sav = 0.d0
    val_choice = -1.d7
    exp_value = 0.d0
    exp_dvalue = 0.d0


    !1. Compute continuation value
    !------------------------------------------------------------------------
    !   
    DO index_y = 1,len_y_grid            !do_1 
      DO index_h_prime = 1,len_h_grid   !do_2
        DO index_a = 1, len_a_grid     !do_3
          !Discounted continuation value and its wealth derivative
          exp_value(index_a,index_h_prime,index_y) = bita*&
              DOT_PRODUCT(pi_y(index_y,:),val(index_a,index_h_prime,:)) 
          exp_dvalue(index_a,index_h_prime,index_y) = bita *&
              DOT_PRODUCT(pi_y(index_y,:),dval(index_a,index_h_prime,:)) 
        END DO          !do_3
      END DO             !do_2
    END DO                !do_1

    !2. Compute policy and value fns
    !-----------------------------------------------------------------------
    val = 0.d0

    DO index_y = 1,len_y_grid  ! d0_4 - Income shock
      DO index_h_prime = 1,len_h_grid	! do_5 -  Chosen housing stock  
        c_node = 1.d-7
        val_node = -1.d7
        h_prime = h_grid(index_h_prime)
        exp_val = exp_value(:,index_h_prime,index_y)
        exp_dval = exp_dvalue(:,index_h_prime,index_y)

        ! 2.1 Compute policy and value fn values and endog. cash-at-hand nodes
        !     given income, housing choice and future income realization
        CALL sub_globalegm( exp_val,exp_dval,z_node,c_node,val_node,k )
        IF (eg_flag==1) THEN
          zend_gr(1:k,index_h_prime,index_y) = z_node(1:k)       
          c_gr(1:k,index_h_prime,index_y) = c_node(1:k)  
          val_gr(1:k,index_h_prime,index_y) = val_node(1:k)
          max_gr(index_h_prime,index_y) = k
        END IF
        

        DO index_h = 1, len_h_grid ! d0_6 - Initial housing stock
          ! 2.2 Recover the policy and value fns on the exogenous grid
          CALL sub_egminterp( k,z_node(1:k),c_node(1:k),val_node(1:k), &
              z_grid(:,index_h_prime,index_h,index_y), &
              con(:,index_h_prime,index_h,index_y), &
              val_choice(:,index_h_prime,index_h,index_y) )

          sav(:,index_h_prime,index_h,index_y) = &
              z_grid(:,index_h_prime,index_h,index_y) - &
              con(:,index_h_prime,index_h,index_y)
        END DO   ! do_6  index_h
      END DO ! do_5  index_h_prime

      ! Solve for housing policy fn 
      DO index_h = 1,len_h_grid 	        
        DO index_a = 1,len_a_grid
          ! Optimal housing choice
          i =  MAXLOC(val_choice(index_a,:,index_h,index_y),1)
          index_house(index_a,index_h,index_y) = i
          h_prime = h_grid(i)
          ! Unconditional Value fn and wealth derivative
          val(index_a,index_h,index_y) = val_choice(index_a,i,index_h,index_y)
          dval(index_a,index_h,index_y) = (1.d0+r)*dfel(con(index_a,i,index_h,index_y))
        END DO
      END DO
    END DO  ! do_4


    !3.Check for convergence
    !-------------------------------
    diff = MAXVAL(ABS(last_val-val))/MAXVAL(ABS(last_val))

    IF (MOD(iter,50)==0) THEN 
      t2 = secnds(0.0)
      PRINT "(' Iteration =', I5,'  with sup_norm =',E14.6,'  secs',F10.3)",&
          iter,diff, t2-t1 
      OPEN(1,FILE='Output.txt',POSITION='APPEND',ACTION='WRITE') 
      WRITE(1,100) iter,diff, t2-t1 
100   FORMAT (' Iteration =', I5,'  with sup_norm =',E14.6,'  secs',F10.3)

      CLOSE(1)
      t1 = t2
    END IF

    IF (diff<toler) EXIT 

  END DO   ! do_0

  IF (iter<10000) THEN

    PRINT *, ' '
    PRINT "(' Final Iteration = ', I6,&
        &'  Successfully converged with sup_norm', E14.6)", iter, diff
    OPEN(1,FILE='Output.txt',POSITION='APPEND',ACTION='WRITE') 
    WRITE(1,110) iter,diff
110 FORMAT (' Final Iteration = ', I6,&
        &'  Successfully converged with sup_norm', E14.6)
    CLOSE(1)

  ENDIF

END SUBROUTINE SUB_VALUEITER



