  SUBROUTINE &
      sub_valueiter(index_house,con,sav,val_choice,val_in)
  ! Iterates on value fn to convergence and outputs saving, con, 
  ! job acceptance and value fns
  ! val_in and index_house: are the value and housing choice fns 
  !                        they are a function of the state      

  ! con, sav and  val_choice:   consumption, saving and value fns
  !                             CONDITIONAL on housing choice.
  !                             They are a function of the state AND
  !                             the current housing choice 

  USE mod_types
  USE mod_common_var  
  USE mod_felicity
  USE mod_int_node

  IMPLICIT NONE

  REAL (dp), INTENT(in), DIMENSION(len_a_grid,len_h_grid,len_y_grid):: val_in
  REAL (dp), INTENT(out), DIMENSION(len_a_grid,len_h_grid,len_h_grid,&
      len_y_grid)      :: con, sav, val_choice
  INTEGER, INTENT(out), DIMENSION(len_a_grid,len_h_grid,len_y_grid):: &
      index_house

  REAL (dp), DIMENSION(len_a_grid,len_h_grid,len_y_grid):: last_val,&
      exp_value,val
  REAL (dp), DIMENSION(len_a_grid,len_h_grid,len_h_grid, len_y_grid):: &
      z_grid
  REAL (dp), DIMENSION(len_a_grid)  :: sav_node, z_node
  REAL (dp) :: cash, diff, h, val_max_cur, val_refinement, y
  REAL (dp), ALLOCATABLE, DIMENSION(:)                  :: c, val2
  REAL(dp), EXTERNAL :: z

  INTEGER   :: index_a, index_h, index_h_prime, index_y, index_z, &
      iter, j, node
  INTEGER   :: flag, i, last_max_a, len, max_loc
  REAL      :: t1, t2


  iter = 0
  diff = 100.d0; sav = 0.d0; val = val_in; val_choice = -1.d7

  t1 = secnds(0.0)	

  ! Create grid for intermediate variable cash at hand
  CALL sub_grid(2,len_a_grid,min_a,max_a,z_node)

  !Compute cash at hand on the exogenous grid once and for all
  !-----------------------------------------------------
  DO index_y = 1,len_y_grid          
    DO index_h = 1,len_h_grid  
      DO index_h_prime = 1, len_h_grid
        DO index_a = 1, len_a_grid    
          z_grid(index_a,index_h_prime,index_h,index_y) = &
              z(a_grid(index_a),index_h_prime,index_h,index_y)
        END DO
      END DO
    END DO
  END DO

  DO iter = 1,10000
    exp_value = 0.d0
    last_val = val

    !1. Compute continuation value
    !---------------------------------------
    !   
    DO index_y = 1,len_y_grid
      DO index_h_prime = 1,len_h_grid    
        DO index_a = 1, len_a_grid         
          !Discounted continuation value
          exp_value(index_a,index_h_prime,index_y) = bita*&
              DOT_PRODUCT(pi_y(index_y,:),val(index_a,index_h_prime,:)) 
        END DO
      END DO
    END DO

    !2. Compute value fns
    !---------------------------------------

    DO index_y = 1,len_y_grid  ! I - Income shock
      DO index_h_prime = 1,len_h_grid ! II -  Chosen housing stock 

        last_max_a = 1 !Location of last optimal saving (for monotonicity)

        DO index_z = 1,len_a_grid  ! III- Financial wealth

          ALLOCATE(c(last_max_a:len_a_grid))
          flag = 1
          cash = z_node(index_z)
          DO j = last_max_a,len_a_grid
            c(j) = cash - a_grid(j)
            IF (c(j).LE.0 .AND. j>1) EXIT
          END DO

          ! Solve for optimal saving given housing choice
          h_prime = h_grid(index_h_prime)
          ALLOCATE(val2(last_max_a:j-1))
          val2 = fel(c(last_max_a:j-1)) + &
              exp_value(last_max_a:j-1,index_h_prime,index_y)
          max_loc = MAXLOC(val2,1) + last_max_a -1
          val_max_cur = val2(max_loc)
          last_max_a = max_loc
          val_grid(index_z,index_h_prime,index_y) = val_max_cur
          sav_node(index_z) = a_grid(max_loc)
          c_grid(index_z,index_h_prime,index_y) = & 
              cash - a_grid(max_loc)
          DEALLOCATE(c,val2)
        END DO  ! III  

        DO index_h = 1,len_h_grid
          DO index_a = 1,len_a_grid
            cash = z_grid(index_a,index_h_prime,index_h,index_y) 
            ! Recover RIGHT interpolating nodes
            CALL sub_int_node(z_node,cash,node)
            node = node - 1
            val_choice(index_a,index_h_prime,index_h,index_y) = &
                val_grid(node,index_h_prime,index_y) + &
                ( val_grid(node+1,index_h_prime,index_y) - &
                val_grid(node,index_h_prime,index_y) )/ &
                ( z_node(node+1) - z_node(node) ) * ( cash - z_node(node) )
            sav(index_a,index_h_prime,index_h,index_y) = &
                sav_node(node) + &
                ( sav_node(node+1) - sav_node(node) )/ &
                ( z_node(node+1) - z_node(node) ) * ( cash - z_node(node) )
            con(index_a,index_h_prime,index_h,index_y) = cash - &
                sav(index_a,index_h_prime,index_h,index_y)
          END DO
        END DO
      END DO  ! II

      DO index_h = 1,len_h_grid
        DO index_a = 1,len_a_grid 
          ! Optimal housing choice
          i =  MAXLOC(val_choice(index_a,:,index_h,index_y),1)
          index_house(index_a,index_h,index_y) = i
          ! Value fn before housing choice
          val(index_a,index_h,index_y) = &
              val_choice(index_a,i,index_h,index_y)
        END DO
      END DO

    END DO  ! I


    !3.Check for convergence
    !-------------------------------
    diff = MAXVAL(ABS(last_val-val))/MAXVAL(ABS(last_val))

    IF (MOD(iter,50)==0) THEN 
      t2 = secnds(0.0)
      PRINT "(' Iteration =', I5,'  with sup_norm =',E14.6,'  secs',F10.3)",&
          iter,diff, t2-t1 
      OPEN(1,FILE='Output.txt',POSITION='APPEND', ACTION='WRITE') 
      WRITE(1,100) iter,diff, t2-t1 
100   FORMAT (' Iteration =', I5,'  with sup_norm =',E14.6,'  secs',F10.3)
      CLOSE(1)
      t1 = t2
    END IF

    IF (diff<toler) EXIT 

  END DO

  IF (iter<10000) THEN

    PRINT *, ' '
    PRINT "(' Final Iteration = ', I6,&
        &'  Successfully converged with sup_norm', E14.6)", iter, diff
    OPEN(1,FILE='Output.txt',POSITION='APPEND',ACTION='WRITE') 
    WRITE(1,110) iter,diff
110 FORMAT (' Final Iteration = ', I6,&
        &'  Successfully converged with sup_norm', E14.6)
    CLOSE(1)
  ENDIF

END SUBROUTINE SUB_VALUEITER



